//========================================================= file = demo.c =====
//=  Program to demonstrate raw send/recv toolkit                             =
//=============================================================================
// Copyright (c) 1999 - 2003
// NetGroup, Politecnico di Torino (Italy)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
// 3. Neither the name of the Politecnico di Torino nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//=============================================================================
//=  Notes:                                                                   =
//=    1) This product includes software developed by the Politecnico         =
//=       di Torino, and its contributors.                                    =
//=    3) Runs on WinXP and WinVista only. A device index has to be defined   =
//=       using the definition of DEVICE_INDEX on this file.                  =
//=    4) A warning is shown when compiling which doesn't affect execution    =
//=---------------------------------------------------------------------------=
//= Example execution:                                                        =
//=                                                                           =
//=   Initializing...                                                         =
//=   Sending...                                                              =
//=   Receiving...                                                            =
//=     Received packet (first 20 bytes of 1514) = 0060CF2051F41122334455660  =
//=       001020304050607                                                     =
//=   Closing the interfaces...                                               =
//=---------------------------------------------------------------------------=
//=  Build:                                                                   =
//=  gcc demo.c rawstuff.c -lwpcap -lws2_32 -L.\Lib -I.\Include -o demo       =
//=---------------------------------------------------------------------------=
//=  Execute: demo                                                            =
//=---------------------------------------------------------------------------=
//=  Authors: Miguel Jimeno                                                   =
//=           Zornitza Genova                                                 =
//=           Kenneth J. Christensen                                          =
//=           University of South Florida                                     =
//=           WWW: http://www.csee.usf.edu/~christen                          =
//=           Email: christen@csee.usf.edu                                    =
//=---------------------------------------------------------------------------=
//=  History: ZG/KJC (10/02/01) - Genesis                                     =
//=           MJ     (01/24/09) - Modified to work with new version of WPcap  =
//=============================================================================
//----- Include files ---------------------------------------------------------
#include "pcap.h"        // Winpcap libraries
#include "remote-ext.h"  // Winpcap libraries

//----- Defines ---------------------------------------------------------------
#define MAX_PKT_LEN   1514   // Maximum packet length
#define NUM_BYTES       40   // Number of bytes in packet
#define INTER_RXTX       2   // Interface to send the packet through

//----- Function prototypes ---------------------------------------------------
int  init_(void);                        // Initialize interfaces
int  recv_(int numInt, int *pkt_len);    // Receive a raw packet
int  send_(int numInt, unsigned char *packet, int pkt_len);  // Send
void close_(void);                       // Cloase all interfaces

//----- Global variables ------------------------------------------------------
unsigned char *recv_buf;     // Global packet receive buffer

//=============================================================================
//=  Main program                                                             =
//=============================================================================
int main()
{
  unsigned char xmit_buf[MAX_PKT_LEN];   // Transmit packet buffer
  int           packet_len;              // Packet length
  int           retcode;                 // Return code
  int           i;                       // Loop counter

  // Initialize all interfaces
  printf("Initializing all interfaces... \n");
  retcode = init_();
  if (retcode < 0)
  {
    printf("*** ERROR - init_() failed (retcode = %d) \n", retcode);
    exit(-1);
  }

  // Build the raw transmit packet
  xmit_buf[0]  = 0x01;          // Destination address
  xmit_buf[1]  = 0x02;
  xmit_buf[2]  = 0x03;
  xmit_buf[3]  = 0x04;
  xmit_buf[4]  = 0x05;
  xmit_buf[5]  = 0x06;
  xmit_buf[6]  = 0x11;          // Source address
  xmit_buf[7]  = 0x22;
  xmit_buf[8]  = 0x33;
  xmit_buf[9]  = 0x44;
  xmit_buf[10] = 0x55;
  xmit_buf[11] = 0x66;
  for(i=12; i<MAX_PKT_LEN; i++)   // Packet body (0, 1, ..., BUF_SIZE - 12)
    xmit_buf[i] = i - 12;

  // Send the raw packet of length MAX_PKT_LEN
  printf("Sending through device %d \n", INTER_RXTX);
  retcode = send_(INTER_RXTX, xmit_buf, MAX_PKT_LEN);
  if (retcode < 0)
  {
    printf("*** ERROR - send_() failed (retcode = %d) \n", retcode);
    exit(-1);
  }

  // Receive the raw packet just sent
  printf("Receiving from %d ... \n", INTER_RXTX);
  retcode = recv_(INTER_RXTX, &packet_len);
  if (retcode == 0)
  {
    printf("*** WARNING - recv_() found nothing in buffers \n");
    exit(-1);
  }
  if (retcode < 0)
  {
    printf("*** ERROR - recv_() failed (retcode = %d) \n", retcode);
    exit(-1);
  }

  // Output the received packet
  if (packet_len > 0)
  {
    if (retcode > 0)
    {
      printf("  Received packet (first %d bytes of %d) = ",
        NUM_BYTES, packet_len);
      for (i=0; i<NUM_BYTES; i++)
        printf("%02X", recv_buf[i]);
      printf("\n");
    }
  }
  else {
    printf("  Zero bytes received... \n");
  }

  // Free memory for receive buffer
  free(recv_buf);

  // Close all interfaces
  printf("Closing all interfaces... \n");
  close_();
}
